@@ -54,10 +54,11 @@ def _iterative_var(arr, axis, keepdims=False):
 
 class BatchNormalizer(Function):
     name = "BatchNormalizer"
-    def __init__(self, axis=0, eps=1e-5):
+    def __init__(self, axis=0, eps=1e-5, memsave=False):
         Function.__init__(self)
         self.axis = axis
         self.eps = eps
+        self.memsave = memsave
         # Cache for backward pass
         self.x_mean = None
         self.x_std = None
@@ -69,7 +70,6 @@ class BatchNormalizer(Function):
         # Scale and shift
         out = std_scaler * x_hat + mean_scaler
         return out
-    
 
     def forward(self, x: xp.ndarray, mean_scaler: xp.ndarray,
                 std_scaler: xp.ndarray) -> xp.ndarray:
@@ -87,19 +87,27 @@ class BatchNormalizer(Function):
             else:
                 self.N = x.shape[self.axis]
         return self._affine(x, self.x_mean, self.x_std, mean_scaler, std_scaler)
+    
+    def _memsave(self, parents, output):
+        parents[0].data = output.data
 
     def backward(self, dout: xp.ndarray) -> tuple:
         x = self.parent_tensors[0].data
+        mean_scaler = self.parent_tensors[1].data
         std_scaler = self.parent_tensors[2].data
+        if self.memsave:
+            x_hat = (x - mean_scaler) / std_scaler
+        else:
+            x_hat = (x - self.x_mean) / self.x_std
         # Gradients w.r.t. scale and bias parameters
         dmean_scaler = None
         dstd_scaler = None
         # Recompute x_hat (memory efficient)
         x_hat = (x - self.x_mean) / self.x_std
         if self.parent_tensors[1].requires_grad:
-            dmean_scaler = xp.sum(dout, axis=self.axis, keepdims=True)
+            dmean_scaler = _reduce(dout, self.axis, xp.sum, keepdims=True)
         if self.parent_tensors[2].requires_grad:
-            dstd_scaler = xp.sum(dout * x_hat, axis=self.axis, keepdims=True)
+            dstd_scaler = _reduce(dout * x_hat, self.axis, xp.sum, keepdims=True)
         # Gradient w.r.t. input x
         dx = None
         if self.parent_tensors[0].requires_grad:
@@ -108,11 +116,11 @@ class BatchNormalizer(Function):
             # Compute gradients using the standard BatchNorm backward formula
             dx = (1.0 / self.N) * (1.0 / self.x_std) * (
                 self.N * dx_hat
-                - xp.sum(dx_hat, axis=self.axis, keepdims=True)
-                - x_hat * xp.sum(dx_hat * x_hat, axis=self.axis, keepdims=True)
+                - _reduce(dx_hat, self.axis, xp.sum, keepdims=True)
+                - x_hat * _reduce(dx_hat * x_hat, self.axis, xp.sum, keepdims=True)
             ) 
         return dx, dmean_scaler, dstd_scaler
 
 
-def batch_normalize(X, mean_scaler, var_scaler, axis, eps=1e-5):
-    return BatchNormalizer(axis, eps)(X, mean_scaler, var_scaler)
\ No newline at end of file
+def batch_normalize(X, mean_scaler, var_scaler, axis, eps=1e-5, memsave=False):
+    return BatchNormalizer(axis, eps, memsave)(X, mean_scaler, var_scaler)
\ No newline at end of file
