@@ -25,7 +25,7 @@ class BatchNorm(Module):
         self.add_buffer("running_mean", ng.zeros((1, num_features), dtype=ng.float32, requires_grad=False))
         self.add_buffer("running_var",  ng.ones((1, num_features), dtype=ng.float32, requires_grad=False))
         # Pre-create the op with the right axes
-        self._bn_op = BatchNormalizer(axis=(0,), eps=self.epsilon)
+        self._bn_op = BatchNormalizer(axis=(0,), eps=self.epsilon, memsave=True)
 
     def forward(self, X):
         from neurograd import Tensor
@@ -62,7 +62,7 @@ class BatchNorm2D(Module):
         self.add_buffer("running_mean", ng.zeros(shape, dtype=ng.float32, requires_grad=False))
         self.add_buffer("running_var",  ng.ones(shape, dtype=ng.float32, requires_grad=False))
         # Axes over which per-channel stats are computed: (N, H, W)
-        self._bn_op = BatchNormalizer(axis=(0, 2, 3), eps=self.epsilon)
+        self._bn_op = BatchNormalizer(axis=(0, 2, 3), eps=self.epsilon, memsave=True)
     
     def forward(self, X):
         from neurograd import Tensor
